@@ -23,13 +23,14 @@ import (
 	"testing"
 	"time"
 
+	"github.com/golang/glog"
 	"github.com/google/cadvisor/client"
 	"github.com/google/cadvisor/client/v2"
-	"github.com/google/cadvisor/integration/common"
 )
 
 var host = flag.String("host", "localhost", "Address of the host being tested")
 var port = flag.Int("port", 8080, "Port of the application on the host being tested")
+var sshOptions = flag.String("ssh-options", "", "Command line options for ssh")
 
 // Integration test framework.
 type Framework interface {
@@ -69,21 +70,10 @@ func New(t *testing.T) Framework {
 	}
 
 	// Try to see if non-localhost hosts are GCE instances.
-	var gceInstanceName string
-	hostname := *host
-	if hostname != "localhost" {
-		gceInstanceName = hostname
-		gceIp, err := common.GetGceIp(hostname)
-		if err == nil {
-			hostname = gceIp
-		}
-	}
-
 	fm := &realFramework{
 		hostname: HostnameInfo{
-			Host:            hostname,
-			Port:            *port,
-			GceInstanceName: gceInstanceName,
+			Host: *host,
+			Port: *port,
 		},
 		t:        t,
 		cleanups: make([]func(), 0),
@@ -98,6 +88,13 @@ func New(t *testing.T) Framework {
 	return fm
 }
 
+const (
+	Aufs         string = "aufs"
+	Overlay      string = "overlay"
+	DeviceMapper string = "devicemapper"
+	Unknown      string = ""
+)
+
 type DockerActions interface {
 	// Run the no-op pause Docker container and return its ID.
 	RunPause() string
@@ -113,6 +110,9 @@ type DockerActions interface {
 	//   -> docker run busybox ping www.google.com
 	Run(args DockerRunArgs, cmd ...string) string
 	RunStress(args DockerRunArgs, cmd ...string) string
+
+	Version() []string
+	StorageDriver() string
 }
 
 type ShellActions interface {
@@ -149,9 +149,8 @@ type dockerActions struct {
 }
 
 type HostnameInfo struct {
-	Host            string
-	Port            int
-	GceInstanceName string
+	Host string
+	Port int
 }
 
 // Returns: http://<host>:<port>/
@@ -241,8 +240,8 @@ type DockerRunArgs struct {
 // RunDockerContainer(DockerRunArgs{Image: "busybox"}, "ping", "www.google.com")
 //   -> docker run busybox ping www.google.com
 func (self dockerActions) Run(args DockerRunArgs, cmd ...string) string {
-	dockerCommand := append(append(append([]string{"docker", "run", "-d"}, args.Args...), args.Image), cmd...)
-
+	dockerCommand := append(append([]string{"docker", "run", "-d"}, args.Args...), args.Image)
+	dockerCommand = append(dockerCommand, cmd...)
 	output, _ := self.fm.Shell().Run("sudo", dockerCommand...)
 
 	// The last line is the container ID.
@@ -254,6 +253,43 @@ func (self dockerActions) Run(args DockerRunArgs, cmd ...string) string {
 	})
 	return containerId
 }
+func (self dockerActions) Version() []string {
+	dockerCommand := []string{"docker", "version", "-f", "'{{.Server.Version}}'"}
+	output, _ := self.fm.Shell().Run("sudo", dockerCommand...)
+	output = strings.TrimSpace(output)
+	ret := strings.Split(output, ".")
+	if len(ret) != 3 {
+		self.fm.T().Fatalf("invalid version %v", output)
+	}
+	return ret
+}
+
+func (self dockerActions) StorageDriver() string {
+	dockerCommand := []string{"docker", "info"}
+	output, _ := self.fm.Shell().Run("sudo", dockerCommand...)
+	if len(output) < 1 {
+		self.fm.T().Fatalf("failed to find docker storage driver - %v", output)
+	}
+	for _, line := range strings.Split(output, "\n") {
+		line = strings.TrimSpace(line)
+		if strings.HasPrefix(line, "Storage Driver: ") {
+			idx := strings.LastIndex(line, ": ") + 2
+			driver := line[idx:]
+			switch driver {
+			case Aufs:
+				return Aufs
+			case Overlay:
+				return Overlay
+			case DeviceMapper:
+				return DeviceMapper
+			default:
+				return Unknown
+			}
+		}
+	}
+	self.fm.T().Fatalf("failed to find docker storage driver from info - %v", output)
+	return Unknown
+}
 
 func (self dockerActions) RunStress(args DockerRunArgs, cmd ...string) string {
 	dockerCommand := append(append(append(append([]string{"docker", "run", "-m=4M", "-d", "-t", "-i"}, args.Args...), args.Image), args.InnerArgs...), cmd...)
@@ -273,6 +309,16 @@ func (self dockerActions) RunStress(args DockerRunArgs, cmd ...string) string {
 	return containerId
 }
 
+func (self shellActions) wrapSsh(command string, args ...string) *exec.Cmd {
+	cmd := []string{self.fm.Hostname().Host, "--", "sh", "-c", "\"", command}
+	cmd = append(cmd, args...)
+	cmd = append(cmd, "\"")
+	if *sshOptions != "" {
+		cmd = append(strings.Split(*sshOptions, " "), cmd...)
+	}
+	return exec.Command("ssh", cmd...)
+}
+
 func (self shellActions) Run(command string, args ...string) (string, string) {
 	var cmd *exec.Cmd
 	if self.fm.Hostname().Host == "localhost" {
@@ -280,13 +326,13 @@ func (self shellActions) Run(command string, args ...string) (string, string) {
 		cmd = exec.Command(command, args...)
 	} else {
 		// We must SSH to the remote machine and run the command.
-		args = append(common.GetGCComputeArgs("ssh", self.fm.Hostname().GceInstanceName, "--", command), args...)
-		cmd = exec.Command("gcloud", args...)
+		cmd = self.wrapSsh(command, args...)
 	}
 	var stdout bytes.Buffer
 	var stderr bytes.Buffer
 	cmd.Stdout = &stdout
 	cmd.Stderr = &stderr
+	glog.Infof("About to run - %v", cmd.Args)
 	err := cmd.Run()
 	if err != nil {
 		self.fm.T().Fatalf("Failed to run %q %v in %q with error: %q. Stdout: %q, Stderr: %s", command, args, self.fm.Hostname().Host, err, stdout.String(), stderr.String())
@@ -302,8 +348,7 @@ func (self shellActions) RunStress(command string, args ...string) (string, stri
 		cmd = exec.Command(command, args...)
 	} else {
 		// We must SSH to the remote machine and run the command.
-		args = append(common.GetGCComputeArgs("ssh", self.fm.Hostname().GceInstanceName, "--", command), args...)
-		cmd = exec.Command("gcloud", args...)
+		cmd = self.wrapSsh(command, args...)
 	}
 	var stdout bytes.Buffer
 	var stderr bytes.Buffer
